<?php

/**
 * Plugin Name:       Post Type Builder (PTB)
 * Plugin URI:        https://themify.me/post-type-builder
 * Description:       This "all-in-one" plugin allows you to create Custom Post Types, Meta Boxes, Taxonomies, and Templates.
 * Version:           2.0.9 
 * Author:            Themify
 * Author URI:        https://themify.me
 * Text Domain:       ptb
 * Domain Path:       /languages
 */
// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}
/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-ptb-activator.php
 */
function activate_ptb() {
    require_once plugin_dir_path(__FILE__) . 'includes/class-ptb-activator.php';
    PTB_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-ptb-deactivator.php
 */
function deactivate_ptb() {
    require_once plugin_dir_path(__FILE__) . 'includes/class-ptb-deactivator.php';
    Ptb_Deactivator::deactivate();
}

function themify_ptb_php_notice() {
    echo '<div class="notice notice-error"><p>' . __( 'Your PHP version is too low, Themify PTB plugin is disabled. Please contact your website administrator and ask them to update PHP to 7.3 or higher for better security and performance.', 'themify-ptb' ) . '</p></div>';
    $network_wide = is_multisite() && is_plugin_active_for_network( 'themify-ptb/themify-ptb.php' );
    deactivate_plugins( 'themify-ptb/themify-ptb.php', true, $network_wide );
}

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_ptb() {
    if ( version_compare( phpversion(), '7.3', '<' ) ) {
        add_action( 'admin_notices', 'themify_ptb_php_notice' );
        return;
    }

    /**
     * The core plugin class that is used to define internationalization,
     * dashboard-specific hooks, and public-facing site hooks.
     */
    require plugin_dir_path(__FILE__) . 'includes/class-ptb.php';

    register_activation_hook(__FILE__, 'activate_ptb');
    register_deactivation_hook(__FILE__, 'deactivate_ptb');
    PTB::get_instance()->set_constants( '2.0.9', plugin_dir_path( __FILE__ ), plugin_dir_url( __FILE__ ) );
    PTB::get_instance()->run();
}
run_ptb();
